#ifndef MOBIUS_DECODER_MFC_H
#define MOBIUS_DECODER_MFC_H

// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Mobius Forensic Toolkit
// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024 Eduardo Aguiar
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
// Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#include <mobius/datetime/datetime.h>
#include <mobius/decoder/data_decoder.h>
#include <mobius/io/file.h>
#include <mobius/bytearray.h>
#include <cstdint>

namespace mobius::decoder
{
class mfc
{
public:
  explicit mfc (const mobius::io::reader&);

  void skip (std::uint64_t);
  bool get_bool ();
  std::uint16_t get_word ();
  std::uint32_t get_dword ();
  std::uint64_t get_qword ();
  std::int32_t get_int ();
  std::uint32_t get_count ();
  std::string get_string ();
  std::string get_guid ();
  std::string get_hex_string (std::uint64_t);
  std::string get_ipv4 ();
  mobius::bytearray get_data (std::uint64_t);
  mobius::datetime::datetime get_ctime ();
  mobius::datetime::datetime get_nt_time ();
  mobius::datetime::datetime get_unix_time ();

private:
  mobius::decoder::data_decoder decoder_;
};

} // namespace mobius::decoder

#endif
